########################################################################
### VMWare RPM Spec File for Kernel Module Source Package
########################################################################



##
## Special macros defined by the VMWare build system.
##
%define   _basename vmware-open-vm-tools
%define   _vm_modules "vmxnet vmhgfs vmblock vmci vsock"
%define   _vmware_version 8.3.18
%define   _vmware_release 975338
%define   _install /usr/lib/vmware-tools/install/vmware-open-vm-tools-kmod-%{kversion}
%define   _vmmemctl_upstreamed 1
%define   _pvscsi_upstreamed 1
%define   _vmxnet3_upstreamed 1

##
## Macros
##



#
# You must pass --define "srpm_rebuild 0" to rpmbuild -bs if you want
# the resulting package to be easily usable on multiple different
# distros/versions.  Basically this macro is used to prevent macro
# expansion at rpmbuild -bs time, while ensuring that macros are used
# for dependency generation and other things at rpmbuild --rebuild time.
#
%{!?srpm_rebuild: %{expand: %%define srpm_rebuild 1}}

#
# Establish which supported distribution is the build host.
#

%define _distro unsupported
%define _distro_rel unknown

%define _rhel 0
%define _sles 0

%define _rhel2 0
%define _rhel3 0
%define _rhel4 0
%define _rhel5 0
%define _rhel6 0

%define _sles8 0
%define _sles9 0
%define _sles10 0


%if "%{?sles_version}" != ""
%define _distro "sles"
%define _sles 1
%endif

%if %{_sles} && "%{sles_version}" == "8"
%define _sles8 1
%endif

%if %{_sles} && "%{sles_version}" == "9"
%define _sles9 1
%endif

%if %{_sles} && "%{sles_version}" == "10"
%define _sles10 1
%endif

%if "%{?_vendor}" == "redhat"
%define _distro "rhel"
%define _rhel 1
# This macro expansion yields _distro_rel to be the version reported by
# lsb_release -r, with any decimal fraction truncated.  E.g. an lsb_release -r
# of "Release:        5.2" becomes a _distro_rel of "5".
%{expand: %%define _distro_rel %(lsb_release -r | awk '{print $2'} | sed -e 's/\..*$//')}
%endif

%if %{_rhel} && "%{_distro_rel}" == "2"
%define _rhel2 1
%endif

%if %{_rhel} && "%{_distro_rel}" == "3"
%define _rhel3 1
%endif

%if %{_rhel} && "%{_distro_rel}" == "4"
%define _rhel4 1
%endif

%if %{_rhel} && "%{_distro_rel}" == "5"
%define _rhel5 1
%endif

%if %{_rhel} && "%{_distro_rel}" == "6"
%define _rhel6 1
%endif

#
# By default, build for the current running kernel.
# The user may specify an alternate target via- e.g.
#   --define "kversion 2.6...."
#
%{!?kversion: %{expand: %%define kversion %(uname -r)}}

%if %{_rhel3} || %{_rhel4} && %{srpm_rebuild}
# These macro expansions divide the kernel version on a RHEL4 system into
# the flavor and non-flavor components.  RHEL4 kernels all end with ".EL"
# followed by the flavor suffix.  E.g. 2.6.9-78.0.1.ELsmp from uname -r
# gets split into 2.6.9-78.0.1.EL for kversion_noflavor, and smp for
# kversion flavor.  This is needed for example because kernel packages
# are named kernel-%{kversion_flavor}-${kversion_noflavor}.
%{expand: %%define kversion_noflavor %(echo %{kversion} | sed -e 's/\(.*\.EL\)\(.*\)$/\1/')}
%endif

%if %{_rhel3} || %{_rhel4} && %{srpm_rebuild} && "%{kversion}" != "%{kversion_noflavor}"
%{expand: %%define kversion_flavor %(echo %{kversion} | sed -e 's/\(.*\.EL\)\(.*\)$/\2/')}
%define kversion_flavor_dash %{kversion_flavor}-
%endif

%if %{_rhel5} && %{srpm_rebuild}
# These macro expansions are the same as above for rhel4, except that RHEL5
# uses a suffix of ".el5" instead of ".EL"
%{expand: %%define kversion_noflavor %(echo %{kversion} | sed -e 's/\(.*\.el5\)\(.*\)$/\1/')}
%endif

%if %{_rhel5} && %{srpm_rebuild} && "%{kversion}" != "%{kversion_noflavor}"
%{expand: %%define kversion_flavor %(echo %{kversion} | sed -e 's/\(.*\.el5\)\(.*\)$/\2/')}
%define kversion_flavor_dash %{kversion_flavor}-
%endif

# RHEL6 kversion is tailed with arch. e.g. "2.6.32-37.el6.x86_64"
# RHEL6 kernel does not seem to have a flavor. If it does have, assume it will
# follow the RHEL5 conviention.
%if %{_rhel6} && %{srpm_rebuild}
%{expand: %%define kversion_noflavor %(echo %{kversion} | sed -e 's/\(.*\.el6\)\(.*\)$/\1/')}
%{expand: %%define kversion_flavor_dash %(echo %{kversion} | sed -e 's/\(.*\.el6\)\(.*\)\(\..*\)$/\2/')-}

%if "%{kversion_flavor_dash}" == "-"
%undefine kversion_flavor_dash
%endif
%endif

%if %{_sles} && %{srpm_rebuild}
# These macro expansions are are similar to the RHEL ones above, except
# that the kernel flavor is deduced from the uname -r string by looking
# at everything after the final '-'.
%{!?kversion_noflavor: %{expand: %%define kversion_noflavor %(echo %{kversion} | sed -e 's/\(.*\)-\([^-]*\)$/\1/')}}
%{!?kversion_flavor: %{expand: %%define kversion_flavor %(echo %{kversion} | sed -e 's/\(.*\)-\([^-]*\)$/\2/')}}
%endif

#
# Establish arch equivalence classes / 'bittedness'.
#
%define all_x86 i386 i486 i586 i686
%define all_x64 x86_64 amd64

%ifarch %{all_x86}
%define bittedness 32
%endif

%ifarch %{all_x64}
%define bittedness 64
%endif

# RHEL3 (Linux 2.4.*) use 'o' for kernel module extention.
%if %{_rhel3}
%define mod_ext o
%else
%define mod_ext ko
%endif

##
## Tags
##

# e.g. open-vm-tools-kmod-2.6.20-15-82663-113164
Name:           %{_basename}-kmod
# i.e. 82663
Version:        %{_vmware_version}
Release:        %{_vmware_release}%{?dist}
Source0:        %{_basename}-kmod-%{version}.tar.gz
BuildRoot:      %{_tmppath}/%{name}-%{version}-%{release}-root-%(%{__id_u} -n)
Packager:       VMware, Inc. <vmware-builds@vmware.com>
Vendor:         VMware, Inc.
License:        GPLv2
URL:            http://packages.vmware.com
Group:          System Environment/Kernel

%if %{_rhel} && %{srpm_rebuild}
# RHEL packages rpmbuild as rpm-build
BuildRequires:  rpm-build
%endif

%if %{_sles} && %{srpm_rebuild}
# SLES packages rpmbuild with rpm
BuildRequires:  rpm
%endif

BuildRequires:  /usr/bin/gcc

%if %{_rhel} && %{_rhel3} == 0 && %{srpm_rebuild}
BuildRequires:  kernel-%{?kversion_flavor_dash}devel = %{kversion_noflavor}
%endif

%if %{_sles} || %{_rhel3} && %{srpm_rebuild}
BuildRequires:  kernel-source = %{kversion_noflavor}
%endif

Summary:        Kernel modules for VMware Tools

%description
Kernel modules for VMware Tools

%prep
%setup -q

%build


echo -e "\n\n\n"
echo -e "STATUS: Building %{name}"
echo -e "STATUS: kversion is %{kversion}"
%if %{srpm_rebuild}
echo -e "STATUS: kversion_noflavor is %{kversion_noflavor}"
echo -e "STATUS: kversion_flavor is %{?kversion_flavor}"
%endif
echo -e "\n\n\n"

for module in "%{_vm_modules}"; do

        echo -e "\n\n\n"
        echo -e "STATUS: Building module ${module}"
        echo -e "\n\n\n"

        if [ "$module" = "vsock" ]; then
                # It's okay if the copy failed: keep going
                cp -f vmci/Module*.symvers vsock/ || echo "COPY FAILED"
        fi
        (cd ${module} && make auto-build VM_UNAME=%{kversion})
done

echo -e "\n\n\n"
echo -e "STATUS: Done building %{name}"
echo -e "\n\n\n"

%install
rm -rf $RPM_BUILD_ROOT
mkdir -p $RPM_BUILD_ROOT

mkdir -p $RPM_BUILD_ROOT/lib/modules/%{kversion}/misc

for module in "%{_vm_modules}"; do

        echo -e "\n\n\n"
        echo -e "STATUS: Installing module ${module}"
        echo -e "\n\n\n"

        cp -v ${module}.o \
                $RPM_BUILD_ROOT/lib/modules/%{kversion}/misc/${module}.%{mod_ext}
done

mkdir -p $RPM_BUILD_ROOT/%{_install}
cp scripts/{installer,functions} $RPM_BUILD_ROOT/%{_install}



%clean
rm -rf $RPM_BUILD_ROOT

##############################################################################

%package %{kversion}

Group:          System Environment/Kernel
Summary:        Kernel modules for VMware Tools

%if %{_rhel} && %{srpm_rebuild}
%if "%{?kversion_flavor}" == ""
Requires:	kernel = %{kversion_noflavor}
%else
Requires:	kernel-%{kversion_flavor} = %{kversion_noflavor}
%endif
%endif

%if %{_sles9} && "%{bittedness}" == "32" && %{srpm_rebuild}
Requires:       /boot/symvers-%{kversion_noflavor}-i386-%{?kversion_flavor}.gz
%endif

%if %{_sles9} && "%{bittedness}" == "64" && %{srpm_rebuild}
Requires:       /boot/symvers-%{kversion_noflavor}-x86_64-%{?kversion_flavor}.gz
%endif

%if %{_sles10} && "%{bittedness}" == "32" && %{srpm_rebuild}
Requires:       /usr/src/linux-%{kversion_noflavor}-obj/i586
%endif

%if %{_sles10} && "%{bittedness}" == "64" && %{srpm_rebuild}
Requires:       /usr/src/linux-%{kversion_noflavor}-obj/x86_64
%endif

Provides:       %{_basename}-kmod-%{_vmware_version}
Conflicts:      VMwareTools

%description %{kversion}
Kernel modules for VMware Tools

# XXX: HACK. This list of files should be dynamically generated, just as
# it is for the ordinary binary RPMs.
%files %{kversion}

%defattr(755,root,root)
/lib/modules/%{kversion}/misc/vmhgfs.%{mod_ext}
%if ! %{_vmmemctl_upstreamed}
/lib/modules/%{kversion}/misc/vmmemctl.%{mod_ext}
%endif
/lib/modules/%{kversion}/misc/vmxnet.%{mod_ext}
/lib/modules/%{kversion}/misc/vmblock.%{mod_ext}
/lib/modules/%{kversion}/misc/vmci.%{mod_ext}
%if ! %{_sles9} && ! %{_rhel3} && ! %{_pvscsi_upstreamed}
/lib/modules/%{kversion}/misc/pvscsi.%{mod_ext}
%endif
%if ! %{_rhel3} && ! %{_vmxnet3_upstreamed}
/lib/modules/%{kversion}/misc/vmxnet3.%{mod_ext}
%endif
/lib/modules/%{kversion}/misc/vsock.%{mod_ext}
%dir /usr/lib/vmware-tools
%dir /usr/lib/vmware-tools/install
%dir %{_install}
%{_install}/installer
%{_install}/functions


%pre %{kversion}

if [ -e "/etc/vmware-tools/locations" ] &&
   [ ! -f "/usr/lib/vmware-tools/dsp" ]; then
        # This is an install or upgrade, and the locations file exists,
        # so presumably another install of tools is present
        cat >&2 <<EOF
VMware Tools cannot install because it appears that another installation of
VMware Tools is already present. Please remove the previous installation and
then attempt to install this copy of VMware Tools again.
EOF
        exit 1
fi

if [ $1 -eq 2 ] && [ -e %{_install}/installer ]; then
   # Upgrade
   %{_install}/installer uninstall
fi

%post %{kversion}
/sbin/depmod -a %{kversion}
sed -i -e "s/@@PACKAGE@@/%{_basename}-kmod-%{kversion}/g" \
       -e "s/@@KERNEL@@/%{kversion}/g" %{_install}/installer
%{_install}/installer install

%preun %{kversion}
if [ $1 -eq 0 ] && [ -e %{_install}/installer ]; then
   # This is an uninstall
   %{_install}/installer uninstall
fi

%postun %{kversion}
/sbin/depmod -a %{kversion}

%changelog


#############################################################################
